 /*---------------------------------------------------------------------------
|                                grebulon.com                                 |
|                    Screen region capture plugin for IrfanView               |
|                                code based on                                |
|       http://www.codeproject.com/KB/gadgets/Screen_Snaper_Gadget.aspx       |
|--------------------------------------+--------------------------------------|
|                                  DCUtility                                  |
|    SolutionName : ScreenSnaperGadget                                        |
|    ProjectName  : Helper                                                    |
|    FileName     : WinCapture.cpp                                            |
|    CreationDate : 2007-03-07                                                |
|    Author       : Dany Cantin                                               |
|    Comments     : dcutility@hotmail.com                                     |
|--------------------------------------+--------------------------------------|
|                             Screen Capture Tool                             |
|                         Windows Vista SideBar Gadget                        |
|--------------------------------------+--------------------------------------|
|                          Copyright  2007 DCUtility                         |
|                              All rights reserved.                           |
 ---------------------------------------------------------------------------*/

#include "stdafx.h"
#include "resource.h"
#include "Helper.h"
#include "DrawUtil.h"
#include "WinCapture.h"
#include "version.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CHelperApp theApp;

BEGIN_MESSAGE_MAP(CHelperApp, CWinApp)
END_MESSAGE_MAP()

HBITMAP CHelperApp::GetImage(eSnapType SnapType)
{
    if (SnapType == CT_DESKTOP || /*SnapType == CT_WINDOW ||*/ SnapType == CT_REGION) {

        CWinCapture Dlg;
        Dlg.SetCaptureMode(SnapType);
        
        if (Dlg.DoModal() != IDCANCEL) {
            CBitmap*    pBitmap = Dlg.GetSnapBitmap();
            if (!pBitmap)
                return NULL;

            HBITMAP hBitmap = (HBITMAP)pBitmap->Detach();
            delete  pBitmap;
            return  hBitmap;
        }
    }

    return NULL;
}

HANDLE CHelperApp::CreateDibFromBitmap(HBITMAP hBitmap)
{
	if (!hBitmap)
		return NULL;

	HDC hdc = CreateCompatibleDC(NULL);
	if (!hdc)
		return NULL;

	HANDLE hRet = NULL;
	BITMAPINFO bmi;

    // init
	memset(&bmi, 0, sizeof(BITMAPINFO));
	bmi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	bmi.bmiHeader.biBitCount = 0;

    // fill BMI infos
	if (GetDIBits(hdc, hBitmap, 0, 0, NULL, &bmi, DIB_RGB_COLORS) != 0 && bmi.bmiHeader.biSizeImage != 0)
	{
		DWORD totalSize = sizeof(BITMAPINFO) + bmi.bmiHeader.biSizeImage;

		if (bmi.bmiHeader.biCompression == BI_BITFIELDS) // add some shit
			totalSize += 12; // 3 DWORDs
		if (bmi.bmiHeader.biClrUsed > 0)
			totalSize += bmi.bmiHeader.biClrUsed * sizeof(RGBQUAD);
//		hRet = GlobalAlloc(GPTR, totalSize); // IRFAN
		hRet = GlobalAlloc(GMEM_FIXED, totalSize);
		if (hRet)
		{
			BITMAPINFO *pBmi = (BITMAPINFO *)hRet;
			memcpy(pBmi, &bmi, sizeof(BITMAPINFO));
			BYTE *pBits = (BYTE *)hRet + sizeof(BITMAPINFO);

            // adjust pixel pointer
            if (bmi.bmiHeader.biCompression == BI_BITFIELDS)
				pBits += 8; // 12 should be correct, bit not working properly !!!???
			if (bmi.bmiHeader.biClrUsed > 0)
				pBits += bmi.bmiHeader.biClrUsed * sizeof(RGBQUAD);

            // set some DPIs
            pBmi->bmiHeader.biXPelsPerMeter = GetDeviceCaps (hdc,LOGPIXELSX);
            pBmi->bmiHeader.biYPelsPerMeter = GetDeviceCaps (hdc,LOGPIXELSY);

            // get pixels
			if (GetDIBits(hdc, hBitmap, 0, abs(bmi.bmiHeader.biHeight), pBits, pBmi, DIB_RGB_COLORS) == 0)
			{
				GlobalFree(hRet);
				hRet = NULL;
			}
		}
	}

	DeleteDC(hdc);
	DeleteObject(hBitmap);
	return hRet;
}

COLORREF CHelperApp::GetColorPick()
{
    CWinCapture Dlg;
    Dlg.SetCaptureMode(CT_COLOR_PICK);
    
    if (Dlg.DoModal() != IDCANCEL)
        return Dlg.GetPickedColor();

    return INVALID_COLOR;
}

LONG CHelperApp::GetMeasure()
{
    CWinCapture Dlg;
    Dlg.SetCaptureMode(CT_MEASURE);
    
    if (Dlg.DoModal() != IDCANCEL)
        return Dlg.GetMeasure();

    return -1;
}

///////////////////////
//
// Operation lists one of the following:
// 1 - CT_REGION
//     Capture a rectangle from the desktop.
//     Return a global memory handle to the captured bitmap, or NULL if canceled.
// 2 - CT_COLOR_PICK
//     Pick a color off the desktop.
//     Return the color of the picked pixel as COLORREF cast to HANDLE,
//     or INVALID_COLOR (-1) if canceled.
// 3 - CT_MEASURE
//     Return the distance between two selected points on the desktop as LONG
//     cast to HANDLE, or -1 if canceled


extern "C" HANDLE GetRegionCapture(int operation, char *textBuffer)
{
    AFX_MANAGE_STATE(AfxGetStaticModuleState());

	switch (operation)
	{
	case CT_COLOR_PICK:
		return (HANDLE)theApp.GetColorPick();
	case CT_MEASURE:
		return (HANDLE)theApp.GetMeasure();
	case CT_REGION:
		HBITMAP hBitmap = theApp.GetImage(CT_REGION);
		return theApp.CreateDibFromBitmap(hBitmap);
	}
	return NULL;
}

///////////////////////
//
// GetColorPick
// Return the color of the picked pixel or INVALID_COLOR (-1) if canceled
extern "C" COLORREF GetColorPick(int param1, char *textBuffer)
{
    AFX_MANAGE_STATE(AfxGetStaticModuleState());

	return theApp.GetColorPick();
}

///////////////////////
//
// GetColorPick
// Return the distance between two selected points on the desktop or -1 if canceled
extern "C" LONG GetMeasure(int param1, char *textBuffer)
{
    AFX_MANAGE_STATE(AfxGetStaticModuleState());

	return theApp.GetMeasure();
}

//////////////////////////
//
//  GetPlugInInfo
//
extern "C" int GetPlugInInfo(char *versionString,char *fileFormats)
//__declspec (dllexport) int GetPlugInInfo(char *versionString,char *fileFormats)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	strcpy(versionString, VERSION_STR);
	strcpy(fileFormats, "Region/custom screen capture");
	return 0;
}
